clear

% where are my custom functions?
addpath('../routines');
addpath('../bsp');

collection=menu('Direkter Lser, Testbeispiele',...
                'unstrukturierte Probleme',...
                'part. Dgln',...
                'Ende');

if collection==1
   loadmat;

   n=size(A,1);
   
   % start flop counter
   flops(0);
   
   % compute sparse cholesky decomposition for A.
   [L,U,P]=lu(A,0.1);
   
   % solve linear system
   b=ones(n,1);
   x=U\(L\(P*b));

   fprintf('Anzahl Fliekommaoperationen: %12.4e\n', flops);
   fprintf('Speicherbedarf LU-Zerlegung:  %12.4e\n', nnz(L)+nnz(U)-n);

elseif collection==2
   % ---   parameters for system size   ---
   % parameter to generate system size n=2^k-1
   k=menu('Problemgre, Testbeispiele',...
          '   49',...
          '  225',...
          '  961',...
          ' 3969',...
          '16129',...
          '65025')+2;

   % system size
   N=2^k-1;
   n=N*N;
   fprintf('size of the problem %12.4e\n',n);
   % --------------------------------------

   nmb=10;
   strg=cell(nmb,1);
   fun=cell(nmb,1);
   strg{1}='laplace 2-D ';                                fun{1}='fun2_1';
   strg{2}='laplace 2-D, alpha=1, beta=100';              fun{2}='fun2_2';
   strg{3}='laplace 2-D, 1+sin(8pix)^2';                  fun{3}='fun2_3';
   strg{4}='laplace 2-D, 1+sin(16pix)^2';                 fun{4}='fun2_4';
   strg{5}='laplace 2-D, 1+sin(32pix)^2';                 fun{5}='fun2_5';
   strg{6}='laplace 2-D, 1+e^{pix}sin(8pix)^2';           fun{6}='fun2_6';
   strg{7}='laplace 2-D, 1+e^{2pix}sin(8pix)^2';          fun{7}='fun2_7';
   strg{8}='laplace 2-D, 1+e^{8pix}sin(8pix)^2';          fun{8}='fun2_8';
   strg{9}='laplace 2-D, spr. Koeff [1,100,1e3,1e4]';    fun{9}='fun2_9';

   strg{nmb}='END';



   choice=menu('Beispielprobleme',strg);...

   if choice<nmb

      fprintf('%s\n',strg{choice});
  
      % depending on the problem generate initial matrix
      A=lap2d(N,fun{choice}); 
      n=size(A,1);
   
      % start flop counter
      flops(0);
   
      % compute minimum degree ordering for A.
      p=symmmd(A);
   
      % compute sparse cholesky decomposition for A.
      U=chol(A(p,p));
   
      % solve linear system
      b=ones(n,1);
      x(p,:)=U\(U'\b(p,:));

      fprintf('Anzahl Fliekommaoperationen:   %12.4e\n', flops);
      fprintf('Speicherbedarf Cholesky Faktor: %12.4e\n', nnz(U));
   
   end % if choice<3
else
   return
end % if-elseif-else
