
   NMB=4;
   STRG=cell(NMB,1);
   param=zeros(NMB,3);
   STRG{1} ='V-cycle, 1 pre, 1 post (smoothing)'; param( 1,:)=[1 1 1];
   STRG{2} ='V-cycle, 2 pre, 2 post (smoothing)'; param( 2,:)=[1 2 2];
   STRG{3} ='W-cycle, 1 pre, 1 post (smoothing)'; param( 3,:)=[2 1 1];
   STRG{4} ='W-cycle, 2 pre, 2 post (smoothing)'; param( 4,:)=[2 2 2];


   type=menu('Multigrid combination',STRG);

   
   global mu nu;
   % number of coarse grid correction steps
   mu=param(type,1);
   % number of pre-smoothing steps
   nu1=param(type,2);
   % number of post-smoothing steps
   nu2=param(type,3);
   % maximum of pre and post-smoothing, needed for CG
   nu=max(nu1,nu2);
   % ------------------------------------------



   
   fprintf('number of smoothing steps: %1d (pre), %1d (post)\n',nu1,nu2);
   fprintf('number of recursive calls: %1d ', mu);
   if mu==1
      fprintf('(V-cycle)');
   elseif mu==2
      fprintf('(W-cycle)');
   end % if-else
   fprintf('\n\n');

   % Choose either Gauss-Seidel or Jacobi smoothing
   global i_smoother;
   
   i_smoother=2; % Gauss-Seidel
   
   
   if nu1~=nu2
      fprintf('number of smoothing steps: %1d (pre), %1d (post)\n',nu,nu);
      fprintf('number of recursive calls: %1d ', mu);
      if mu==1
	 fprintf('(V-cycle)');
      elseif mu==2
	 fprintf('(W-cycle)');
      end % if-else
      fprintf('\n');
   end % if


   % initial guess for the eigenvector
   v0=ones(n,1);
   % normalize v
   v0=v0/norm(v0,2);
   

   % k: number of desired eigenvalues
   for k=1:6
   
       fprintf('\n\nStart Implicitly Restarted Lanczos for A\n');
       OPTIONS.tol=1e-6;
       OPTIONS.stagtol=0;
       OPTIONS.p=max(6,2*k);
       OPTIONS.maxit=300;
       OPTIONS.disp=0; % min(k,20);
       OPTIONS.issym=1;
       OPTIONS.cheb=0;
       OPTIONS.v0=v0;
   
       flops(0);
       [V,D,FLAG] = eigs('Ax0',size(Alist{1},1),k,'sm',OPTIONS); 
       fprintf('flops =%8.1e\n',flops);
       lambda=diag(D);
       [lambda,I]=sort(lambda);
       V=V(:,I);
       
       figure(1); clf;
       for j=1:k
	   v=V(:,j);
	   subplot(3,2,j);
	   
	   plot(1:length(v),v,'-k');
	   title('eigenvector');
       end
       fprintf('computed lambda=%12.8e\n',lambda);
       pause(0.01)
   
   
       fprintf('\nStart Implicitly Restarted Lanczos for A^{-1} using Cholesky\n');
       
       OPTIONS.tol=1e-6;
       OPTIONS.stagtol=0;
       OPTIONS.p=max(6,2*k);
       OPTIONS.maxit=300;
       OPTIONS.disp=0; % min(k,20);
       OPTIONS.issym=1;
       OPTIONS.cheb=0;
       OPTIONS.v0=v0;
       

       flops(0);
       [V,D,FLAG] = eigs('Ax1',size(Alist{1},1),k,'lm',OPTIONS);
       fprintf('flops =%8.1e + %8.1e (decomposition)\n',flops,flpdirect);
       lambda=1./diag(D);
       [lambda,I]=sort(lambda);
       V=V(:,I);
       
       figure(2); clf;
       for j=1:k
	   v=V(:,j);
	   subplot(3,2,j);
	   
	   plot(1:length(v),v,'-b');
	   title('eigenvector');
       end
       title('eigenvector');
       fprintf('computed lambda=%12.8e\n',lambda);
       pause(0.01)
       
       
       fprintf('\nStart Implicitly Restarted Lanczos for A^{-1} using Multigrid\n');
       
       OPTIONS.tol=1e-6;
       OPTIONS.stagtol=0;
       OPTIONS.p=max(6,2*k);
       OPTIONS.maxit=300;
       OPTIONS.disp=0; % min(k,20);
       OPTIONS.issym=1;
       OPTIONS.cheb=0;
       OPTIONS.v0=v0;
       

       flops(0);
       [V,D,FLAG] = eigs('Ax2',size(Alist{1},1),k,'lm',OPTIONS);
       fprintf('flops =%8.1e + %8.1e (MG setup)\n',flops,flpmg);
       lambda=1./diag(D);
       [lambda,I]=sort(lambda);
       V=V(:,I);
       
       figure(3); clf;
       for j=1:k
	   v=V(:,j);
	   subplot(3,2,j);
	   
	   plot(1:length(v),v,'-r');
	   title('eigenvector');
       end
       title('eigenvector');
       fprintf('computed lambda=%12.8e\n',lambda);
       pause(0.01)
  end % for k
