clear

% where are my custom functions?
addpath('../routines');
addpath('../bsp');


% ---   parameters for system size   ---
% parameter to generate system size n=2^k-1
k=menu('Problemgre, Testbeispiele',...
       '   49',...
       '  225',...
       '  961',...
       ' 3969',...
       '16129',...
       '65025')+2;

% system size
N=2^k-1;
n=N*N;
fprintf('size of the system: %d\n',n);
% --------------------------------------


% ---   parameters for multigrid cycle   ---
% number of levels, <=k
level=k-(k-2);
% ------------------------------------------




nmb=10;
strg=cell(nmb,1);
fun=cell(nmb,1);
strg{1}='laplace 2-D ';                                fun{1}='fun2_1';
strg{2}='laplace 2-D, alpha=1, beta=100';              fun{2}='fun2_2';
strg{3}='laplace 2-D, 1+sin(8pix)^2';                  fun{3}='fun2_3';
strg{4}='laplace 2-D, 1+sin(16pix)^2';                 fun{4}='fun2_4';
strg{5}='laplace 2-D, 1+sin(32pix)^2';                 fun{5}='fun2_5';
strg{6}='laplace 2-D, 1+e^{pix}sin(8pix)^2';           fun{6}='fun2_6';
strg{7}='laplace 2-D, 1+e^{2pix}sin(8pix)^2';          fun{7}='fun2_7';
strg{8}='laplace 2-D, 1+e^{8pix}sin(8pix)^2';          fun{8}='fun2_8';
strg{9}='laplace 2-D, jump. coeff [1,100,1e3,1e4]';    fun{9}='fun2_9';

strg{nmb}='END';



choice=menu('choice of sample problems',strg);...

if choice<nmb

   fprintf('%s\n',strg{choice});
   fprintf('number of levels: %d\n',level);

   if choice<=9
      IsSPD=-1;
   else
      IsSPD=0;
   end % if-else
   
   % list of matrices for any level
   Alist=cell(1,level);
   % projection (prolongation and restriction^T)
   Plist=cell(1,level-1);

   % depending on the problem generate initial matrix
   if choice==1
      T=spdiags(ones(N,1)*[-1 2 -1],-1:1,N,N);
      Alist{1}=kron(T,speye(N))+kron(speye(N),T);
   else
      Alist{1}=lap2d(N,fun{choice}); 
   end % if-else



   % --------------   Generation of the multilevel structure   --------------
   % dampening parameter for diagonal smoothing
   omegaD=2/3;
   nn=N;
   l=N+1;
   for m=1:level-1   
       % standard diagonal preconditioning
       Dlist{m}=omegaD*spdiags(1./diag(Alist{m}),0,nn*nn,nn*nn);
       GSFlist{m}=tril(Alist{m});
       GSBlist{m}=triu(Alist{m});
    
       % projection operator
       %                             [1/8  1/4  1/8]
       % 1. compact 9 point stencil  [1/4  1/2  1/4]
       %                             [1/8  1/4  1/8]
       % buff1=spdiags(ones(nn,1)*[1/4 1/4 1/4],-1:1,nn,nn); 
       % buff2=spdiags(ones(nn,1)*[1  0  1],-1:1,nn,nn); 
       % buff =kron(buff1,speye(nn))+kron(speye(nn),buff1)...
       %      +(1/8)*kron(buff2,buff2);
       %                             [ 0   1/2  1/2]
       % 2. compact 7 point stencil  [1/2   1   1/2]
       %                             [1/2  1/2   0 ]
       buff1=spdiags(ones(nn,1)*[1/2 1/2 1/2],-1:1,nn,nn); 
       buff2=spdiags(ones(nn,1)*[ 0   0   1], -1:1,nn,nn); 
       buff =kron(buff1,speye(nn))+kron(speye(nn),buff1)...
            +(1/2)*(kron(buff2,buff2)+kron(buff2',buff2'));
       % coarse grid, mesh size divided by 2 in any space direction
       crs=(2:2:nn)'*ones(1,(nn-1)/2,1)+ones((nn-1)/2,1)*(nn:2*nn:nn*nn-1);
       crs=crs(:);
       Plist{m}=buff(:,crs);
          
       l=l/2;
       nn=nn-l;
       % coarse grid system
       % Galerking-type
       Alist{m+1}=Plist{m}'*Alist{m}*Plist{m};
       % Geometric type
       % Alist{m+1}=lap2d(nn,fun{choice}); 
     
   end % for m
   if IsSPD
      Dlist{level}.U=chol(Alist{level});Dlist{level}.L=Dlist{level}.U';
   else
      [Dlist{level}.L,Dlist{level}.U]=lu(Alist{level});
   end % if-else
   GSFlist{level}.L=Dlist{level}.L;
   GSFlist{level}.U=Dlist{level}.U;
   GSBlist{level}.L=Dlist{level}.L;
   GSBlist{level}.U=Dlist{level}.U;
   % ------------   END Generation of the multilevel structure   ------------

   
   % call script file with iteration process
   iteration;
   
end % if choice<3
