function [c, s] = GIVENS(a,b)
% [c, s] = GIVENS(a, b)
%
% computes parameter for Givens rotation
% U = [ c, s';
%      -s, c'],
% such that U' * [a; b] is a rotation onto the first unit vector


if abs(b) <= 2*eps*abs(a)
   c = 1;
   s = 0;
else
   if abs(b) > abs(a)
      t = -a/b;
      s = 1 / sqrt(1 + t'*t);
      c = s*t;
   else
      t = -b/a;
      c = 1 / sqrt(1 + t'*t);
      s = c*t;
   end % if-else
end % if-else
