function [v,beta,mu] = HOUSE(x)
% v = HOUSE(x)
% [v,beta] = HOUSE(x)
% [v,beta,mu] = HOUSE(x)
%
% computes a vector v used for the  Householder transformation on x.
% Reflection onto a multiple of the first unit vector, i.e., 
% (I - (2/v'*v) * v*v') * x = norm(x) * e1


n = length(x); 
mu = norm(x,2);
v = x;
if mu ~= 0
   if (x(1)>=0)
      beta=x(1)+mu;
   else
      beta=x(1)-mu;
   end % if-else
   v(2:n) = v(2:n) / beta;
   beta=-abs(beta)/mu;
else 
   beta=-2;   
end % if-else
v(1) = 1;

