function y=MGCall(b,x,Alist,Plist,Rlist,nu1,nu2,mu,Smoother,Llist,Smoother2,Llist2)
% y=MGCall(b,x,Alist,Plist,Rlist,nu1,nu2,mu,Smoother,Llist)
%
% recursive multigrid call
%
% --------------------------------------------------------------------------
%
% Input
% -----
% b         right hand side on the finest level
% x         initial solution on the finest level
% Alist     cell array of system matrices for any level
% Plist     cell array of prolongation operators
% Rlist     cell array of restriction  operators (transposed!)
% nu1       number of pre-smoothing steps
% nu2       number of post-smoothing steps
% Smoother  function name of the Smoother
% Llist     parameter list for the smoother. The smoother is called by
%           feval(Smoother,b,x,Llist)
% Smoother2 OPTIONAL, if nu1>0 and nu2>0, Smoother refers to pre-smoothing
%           Smoother2 refers to post-smoothing
% Llist2    OPTIONAL, parameter list for the post-smoother. The post-smoother
%           is called by feval(Smoother2,b,x,Llist2)
% 
% --------------------------------------------------------------------------
%
% Output
% ------
% y         approximate solution obtained by one multigrid step


m=length(Alist);
y=x;
r=zeros(size(x,1),size(x,2));

if m==1
   % direct solution on the coarsest grid
   y=(Llist{m}.U)\ ((Llist{m}.L)\b);
else   

   % pre-smoothing
   for l=1:nu1
       r=b-Alist{1}*y;
       y=y+feval(Smoother,r,r,Llist{1});
   end % for

   % coarse grid correction  
   v=2:m-1;
   R=zeros(size(Alist{2},1),size(x,2));
   if nargin==12
      for l=1:mu
          R=Rlist{1}'*(b-Alist{1}*y);
          X=zeros(size(Alist{2},1),size(x,2));
          y=y+Plist{1}*MGCall(R,X,Alist([v m]),Plist(v),Rlist(v),nu1,nu2,mu,...
	                      Smoother,Llist([v m]),Smoother2,Llist2([v m]));
      end % for l
      % post-smoothing
      for l=1:nu2
	  r=b-Alist{1}*y;
	  y=y+feval(Smoother2,r,r,Llist2{1});
      end % for
   else   
      for l=1:mu
          R=Rlist{1}'*(b-Alist{1}*y);
          X=zeros(size(Alist{2},1),size(x,2));
          y=y+Plist{1}*MGCall(R,X,Alist([v m]),Plist(v),Rlist(v),nu1,nu2,mu,...
	                      Smoother,Llist([v m]));
      end % for l
      % post-smoothing
      for l=1:nu2
	  r=b-Alist{1}*y;
	  y=y+feval(Smoother,r,r,Llist{1});
      end % for
   end % if-else
   
end % if-else
